{-# LANGUAGE OverloadedStrings #-}

module Lisa
    ( app
    , mkConfig
    ) where

import Control.Monad.IO.Class (MonadIO)
import Data.Maybe (fromMaybe)
import Text.Blaze.Html (Html)
import Text.Blaze.Html.Renderer.Utf8 (renderHtml)
import Web.Spock (SpockM, ActionCtxT, get, lazyBytes, param, readSession, request, root, setHeader, var, (<//>))
import Web.Spock.Config (PoolOrConn(PCNoDatabase), SpockCfg, defaultSpockCfg)

import Lisa.Types (Session, State, emtpySession, mkState, getAllLecturesCached)

import qualified Lisa.Squeak as Squeak
import qualified Lisa.Views as Views

-- Utilities

-- From https://github.com/Rockfordal/spockmini/blob/51af99760bbf68c956e906c9633182700a18be9c/src/Web/Utils.hs#L17
blaze :: MonadIO m => Html -> ActionCtxT ctx m a
blaze html = do
    setHeader "Content-Type" "text/html; charset=utf-8"
    lazyBytes $ renderHtml html

-- Exports

mkConfig :: IO (SpockCfg () Session State)
mkConfig = do
    state <- mkState
    defaultSpockCfg emtpySession PCNoDatabase state

app :: SpockM () Session State ()
app = do
    get root $ do
        blaze Views.viewIndex
    
    get ("lectures" <//> var) $ \lid -> do
        documents <- Squeak.getDocumentsByLectureId lid
        blaze $ either Views.viewSqueakError (Views.viewLecture lid) documents

    get "lectures" $ do
        query <- fromMaybe "" <$> param "query"
        lectures <- getAllLecturesCached
        blaze $ either Views.viewSqueakError (Views.viewLectures query) lectures

    get "debug" $ do
        req <- request
        sess <- readSession
        blaze $ Views.viewDebug req sess
